/* -*- Mode: c++ -*-
 * $Id: s.xotclTrace.c 1.5 01/10/11 18:37:46+02:00 neumann@mohegan.wu-wien.ac.at $
 *  
 *  Extended Object Tcl (XOTcl)
 *
 *  Copyright (C) 1999-2001 Gustaf Neumann, Uwe Zdun
 *
 *
 *  findXOTcl.h -- 
 *  
 *  Find XOTcl library location for package require and put it into the auto_path
 *  
 */

#include "tclInt.h"
#include "xotclInt.h"

#define isXOTclDirString(m) (\
	*m   == '/' && m[1] == 'x' && m[2] == 'o' && m[3] == 't' && \
	m[4] == 'c' && m[5] == 'l' && m[6] == '-')
#define isXOTclFullDirString(m) (\
	m[7] == 'f' && m[8] == 'u' && m[9] == 'l' && \
	m[10] == 'l' && m[11] == '-')

int correctAutoPath(Tcl_Interp* in) {
  Tcl_DString ds, topDir, version, cmd;
  char *dir, *d, *beginVersion, *topDirStr = 0;
  int topDirCnt = 0, versionCount=0;
  int dsLength;

#ifdef PRE81
  d = TclGetCwd(in);
  if (d == NULL) {
    return TCL_ERROR;
  }
  Tcl_DStringInit(&ds);
  Tcl_DStringAppend(&ds, d, -1);
#else

  if (Tcl_GetCwd(in, &ds) == NULL) {
    return TCL_ERROR;
  }
#endif
  dsLength = Tcl_DStringLength(&ds);
  d = dir = Tcl_DStringValue(&ds);

  Tcl_DStringInit(&cmd);

  while (topDirCnt < dsLength) {
    if (isXOTclDirString(d)) {
      /* which version number ?*/
      Tcl_DStringInit(&version);
      Tcl_DStringInit(&topDir);


      if (isXOTclFullDirString(d)) {
	d+=12; topDirCnt+=12;
      } else {
	/* ok top is above the XOTcl dir */
	Tcl_DStringAppend(&topDir, dir, topDirCnt);
	topDirStr = Tcl_DStringValue(&topDir);
	d+=7; topDirCnt+=7;
      }
      beginVersion = d;
      /* search next '/', if any 
	 under win "while (d && *d != '/')" does not work,
	 therefore we use the length
      */

      while (topDirCnt < dsLength && *d != '/') {
	topDirCnt++; d++; versionCount++;
      }
      Tcl_DStringAppend(&version, beginVersion, versionCount);
      
      if (!topDirStr) {
	Tcl_DStringAppend(&topDir, dir, topDirCnt);
	topDirStr = Tcl_DStringValue(&topDir);
      }

      /*
       * ok, we have the dir above xotcl-<Version> in topdir ... now find the 
       * append the lib lib, where package index is
       */
      Tcl_DStringAppend(&topDir, "/xotcl-", 7);
      Tcl_DStringAppend(&topDir, 
			Tcl_DStringValue(&version),
			Tcl_DStringLength(&version));

      Tcl_DStringAppend(&cmd, topDirStr, -1);
      Tcl_DStringAppend(&cmd, " ", 1);

      Tcl_DStringFree(&version);
      Tcl_DStringFree(&topDir);


      break;
    }
    topDirCnt++; d++;
  }

  Tcl_DStringAppend(&cmd, XOLIBPKG, -1);
  /*Tcl_DStringAppend(&cmd, "/libxotcl", -1);*/
  Tcl_DStringAppend(&cmd, " ", 1);

  Tcl_DStringAppend(&cmd, Tcl_GetVar(in, "auto_path", TCL_GLOBAL_ONLY), -1);

  Tcl_SetVar(in, "auto_path", Tcl_DStringValue(&cmd), TCL_GLOBAL_ONLY);
   
  /*fprintf(stderr, "auto_path in findXOTcl= %s\n", Tcl_DStringValue(&cmd));*/

  Tcl_DStringFree(&cmd);
  Tcl_DStringFree(&ds);
}
